// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the GNU LGPL license.

function [ evalf , u ] = intprb_ldgen ( varargin )
  // Returns uniform numbers from a low discrepancy sequence.
  //
  // Calling Sequence
  //   [ evalf , u ] = intprb_ldgen ( callf , n )
  //   [ evalf , u ] = intprb_ldgen ( callf , n , ldseq )
  //   [ evalf , u ] = intprb_ldgen ( callf , n , ldseq , strict )
  //   [ evalf , u ] = intprb_ldgen ( callf , n , ldseq , strict , verbose )
  //
  // Parameters
  //   callf : a 1 x 1 matrix of floating point integers, the number of calls to the function.
  //   n: a 1 x 1 matrix of floating point integers, the spatial dimension.
  //   ldseq : a 1 x 1 matrix of strings, the name of the sequence. The list of available sequences can be computed by lowdisc_methods(). 
  //   strict : a 1 x 1 matrix of boolean, set to %t to make so that evalf==callf. (default = %f)
  //   verbose : a 1 x 1 matrix of boolean, set to %t to enable verbose messages. (default = %f)
  //   evalf : a 1 x 1 matrix of floating point integers, the actual number of function evaluations. We have evalf >= callf.
  //   u : a evalf x n matrix of doubles, the uniform random numbers in [0,1]^n.
  //
  // Description
  // In dimension n, generate more than callf experiments with
  // low discrepancy sequence ldseq.
  //
  // The "lowdisc" module must be installed to use this function.
  // To install this module, run 
  //
  // atomsInstall("lowdisc")
  //
  // then restart Scilab.
  //
  // Returns the number of suggested function evaluations evalf
  // and the uniform numbers u in [0,1]^n.
  //
  // If strict is false, then the optimum number of simulations evalf is
  // used. If strict is true, the number of simulations is equal to 
  // the required one, that is, we have evalf == callf.
  //
  // When the optimum number of simulations is used, 
  // the value of evalf might be extremely large.
  // Indeed, the rule of thumb is that the asymptotic rate 
  // of a low discrepancy sequence is reached when the number of
  // experiments is greater than b^n, where b is the base of the 
  // sequence and n is the dimension of the space.
  // If the generated number of experiments is too large, we should 
  // set strict to %t.
  //
  // TODO : remove this function when it is integrated to the lowdisc module.
  //
  // Examples
  // // Generate more than 100 points from a Halton sequence in dimension 4
  // callf = 100
  // n = 4
  // ldseq = "halton"
  // [ evalf , u ] = intprb_ldgen ( callf , n , ldseq )
  //
  // Authors
  //   Michael Baudin - 2010 - DIGITEO
  
  [lhs, rhs] = argn();
  apifun_checkrhs ( "intprb_ldgen" , rhs , 2 : 5 )
  apifun_checklhs ( "intprb_ldgen" , lhs , 0 : 2 )
  //
  callf = varargin(1)
  n = varargin(2)
  ldseq = "sobolf"
  if ( rhs >= 3 ) then
    if ( varargin(3) <> [] ) then
      ldseq = varargin(3)
    end
  end
  strict = %f
  if ( rhs >= 4 ) then
    if ( varargin(4) <> [] ) then
      strict = varargin(4)
    end
  end
  verbose = %f
  if ( rhs >= 5 ) then
    if ( varargin(5) <> [] ) then
      verbose = varargin(5)
    end
  end
  //
  // Check input arguments
  apifun_checktype ( "intprb_ldgen" , callf ,   "callf" ,   1 , "constant" )
  apifun_checktype ( "intprb_ldgen" , n ,       "n" ,       2 , "constant" )
  apifun_checktype ( "intprb_ldgen" , ldseq ,   "ldseq" ,   3 , "string" )
  apifun_checktype ( "intprb_ldgen" , strict ,  "strict" ,  4 , "boolean" )
  apifun_checktype ( "intprb_ldgen" , verbose , "verbose" , 5 , "boolean" )
  //
  apifun_checkscalar ( "intprb_ldgen" , callf ,    "callf" ,    1 )
  apifun_checkscalar ( "intprb_ldgen" , n ,        "n" ,        2 )
  apifun_checkscalar ( "intprb_ldgen" , ldseq ,    "ldseq" ,    3 )
  apifun_checkscalar ( "intprb_ldgen" , strict ,   "strict" ,   4 )
  apifun_checkscalar ( "intprb_ldgen" , verbose ,  "verbose" ,  5 )
  //
  // Check that the lowdisc module is there
  isld = atomsIsLoaded("lowdisc")
  if ( ~isld ) then
    errmsg = msprintf(gettext("%s: The Low Discrepancy module is not installed. Type atomsInstall(""lowdisc"") to install this module, then restart Scilab."), "intprb_crudeld");
    error(errmsg)
  end
  //
  lds = lowdisc_new(ldseq);
  lds = lowdisc_configure(lds,"-dimension",n);
  select ldseq
  case "fauref"
    base = lowdisc_get(lds,"-faurefprime");
  case "faure"
    base = lowdisc_get(lds,"-faureprime");
  case "halton"
    // Nothing to do
  case "haltonf"
    // Nothing to do
  case "niederreiter-base-2"
    base = 2
  case "niederreiterf"
    base = lowdisc_niederbase ( n );
  case "reversehalton"
    // Nothing to do
  case "reversehaltonf"
    // Nothing to do
  case "sobol"
    // Nothing to do
  case "sobolf"
    // Nothing to do
  else
    lds = lowdisc_destroy(lds);
    msg = msprintf(gettext("%s: Cannot compute base: Unexpected sequence %s.") , "intprb_ldgen" ,ldseq);
    error ( msg )
  end
  if ( strict ) then
    evalf = callf
    skip = 0
    leap = 0
  else
    if ( or ( ldseq == [ "faure" "fauref" ] ) ) then
      [evalf,skip,leap] = lowdisc_fauresuggest ( n , base , callf )
    elseif ( or ( ldseq == [ "halton" "haltonf" ] ) ) then
      [evalf,skip,leap] = lowdisc_haltonsuggest ( n , callf )
    elseif ( or ( ldseq == [ "niederreiter-base-2" "niederreiterf" ] ) ) then
      [evalf,skip,leap] = lowdisc_niedersuggest ( n , base , callf )
    elseif ( or ( ldseq == [ "reversehalton" "reversehaltonf" ] ) ) then
      // No parameter
      evalf = callf
      skip = 0
      leap = 0
    elseif ( or ( ldseq == [ "sobol" "sobolf" ] ) ) then
      // Do not use lowdisc_sobolsuggest which suggests excessively 
      // large number of simulations.
      evalf = 2^ceil(log2(callf))
      skip = 0
      leap = 0
    else
      lds = lowdisc_destroy(lds);
      msg = msprintf(gettext("%s: Cannot compute [evalf,skip,leap] Unexpected sequence %s.") , "intprb_ldgen",ldseq);
      error ( msg )
    end
  end
  if ( verbose ) then
    mprintf(gettext("%s: Ldseq = %s.\n"), "intprb_ldgen" , string(ldseq) );
    mprintf(gettext("%s: Evalf = %s.\n"), "intprb_ldgen" , string(evalf) );
    mprintf(gettext("%s: Skip = %s.\n"), "intprb_ldgen" , string(skip) );
    mprintf(gettext("%s: Leap = %s.\n"), "intprb_ldgen" , string(leap) );
    mprintf(gettext("%s: Strict = %s.\n"), "intprb_ldgen" , string(strict) );
  end
  lds = lowdisc_configure(lds,"-skip",skip);
  lds = lowdisc_configure(lds,"-leap",leap);
  lds = lowdisc_startup (lds);
  [lds,u]=lowdisc_next(lds,evalf);
  lds = lowdisc_destroy(lds);
endfunction


